//
#include "d3d.h"
#include "hardware.h"

Point::Point() {
	set_xyz(0,0,0);
	pos[3]=1;
};
Point::~Point(){};
	
//----------------------------------------------------------------------

void Point::set_xyz(float inx, float iny, float inz) {
	pos[0]=inx;
	pos[1]=iny;
	pos[2]=inz;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------

d3Vector::d3Vector() {
	set_dir(0,0,1);
	dir[3]=1;
};
d3Vector::~d3Vector(){};
	
//----------------------------------------------------------------------

void d3Vector::set_dir(float inx, float iny, float inz) {
	dir[0]=inx;
	dir[1]=iny;
	dir[2]=inz;
}

//----------------------------------------------------------------------

void d3Vector::normalize(void) {
	float len=length();
	if (len<.0001f) len=.0001f;
	dir[0]/=len;
	dir[1]/=len;
	dir[2]/=len;
}

//----------------------------------------------------------------------

float d3Vector::length(void) {
	return (float) sqrt(dir[0]*dir[0]+
		                 dir[1]*dir[1]+
							  dir[2]*dir[2]);
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------

Vertex::Vertex() {
	set_color(1,1,1);
	color[3]=1;
	set_texture(0,0);
};
Vertex::~Vertex(){};

//----------------------------------------------------------------------

void Vertex::set_color(float inr, float ing, float inb) {
	color[0]=inr;
	color[1]=ing;
	color[2]=inb;
}

//----------------------------------------------------------------------

void Vertex::set_texture(float inx, float iny) {
	texture[0]=inx;
	texture[1]=iny;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------

Face::Face() {
	nextface=0;
	set_color(1,1,1);
	color[3]=1;
};
Face::~Face(){};

//----------------------------------------------------------------------

void Face::set_color(float inr, float ing, float inb) {
	color[0]=inr;
	color[1]=ing;
	color[2]=inb;
}

//----------------------------------------------------------------------

void Face::draw(void) {
	glBegin(GL_QUADS);
		glTexCoord2fv(&vertexs[0].texture[0]); glColor3fv(&vertexs[0].color[0]); glVertex3fv(&vertexs[0].pos[0]);
		glTexCoord2fv(&vertexs[1].texture[0]); glColor3fv(&vertexs[1].color[0]); glVertex3fv(&vertexs[1].pos[0]);
		glTexCoord2fv(&vertexs[2].texture[0]); glColor3fv(&vertexs[2].color[0]); glVertex3fv(&vertexs[2].pos[0]);
		glTexCoord2fv(&vertexs[3].texture[0]); glColor3fv(&vertexs[3].color[0]); glVertex3fv(&vertexs[3].pos[0]);
		glEnd();
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------

Object3d::Object3d() {
	face=0;
	nfaces=0;
	set_color(1,1,1);
	color[3]=1;
	set_xyz(0,0,0);
	pos[3]=1; hd=0; pt=0;
};

Object3d::~Object3d(){
	Face *ptr=face,*t;
		while(ptr) {
			t=ptr->nextface;
			delete ptr;
			ptr=t;
		}
	nfaces=0;
};

//----------------------------------------------------------------------

void Object3d::set_color(float inr, float ing, float inb) {
	color[0]=inr;
	color[1]=ing;
	color[2]=inb;
}

//----------------------------------------------------------------------

void Object3d::set_xyz(float inx, float iny, float inz) {
	pos[0]=inx;
	pos[1]=iny;
	pos[2]=inz;
}

//----------------------------------------------------------------------

void Object3d::add_face(Face *inface) {
	if (face==0) {
		face=new Face;
		if (face==0) bail("Can't make 1st new object3d");
		*face=*inface;
	}
	else {
		Face *ptr=face;
		while (ptr->nextface!=0) ptr=ptr->nextface;
		ptr->nextface=new Face;
		if (ptr->nextface==0) bail("Can't make new object3d");
		*ptr->nextface=*inface;
	}
	nfaces++;
}

//----------------------------------------------------------

void Object3d::draw(float size) {
	Face *ptr=face;

	while(ptr) {
		ptr->draw();
		ptr=ptr->nextface;
	}
	Hw::get()->pref.ntris+=nfaces*2;
}

//----------------------------------------------------------------------

