#if !defined dialog_h
#define      dialog_h

#include "base.h"
#include "main.h"
#include "camera.h"
#include "hardware.h"
#include "mouse.h"
#include "text.h"
#include "key.h"
#include "screen.h"
#include "sound.h"

class Texture;

enum dialognames {THIS,MAIN,OPTIONS,SAVE,LOAD,GENERAL,
                  COLOR,LIGHTING,SOUND,LIGHTMAP,RANDOMS,
						CAMERA,SCREEN};

//---------------------------------------------------------------------------------

class Widget:public Base{
public:     Widget();
	virtual ~Widget();

	static int textSize;
	static Texture *tex;
	static Text *text;

	char label[80];
	int ID, index, type;
	float width,height, x2,y2;

	virtual void init(char *s, int inID, int inindex, int intype,
		     float inx, float iny, float inwidth, float inheight,
			  float invalue=0, float inmin=0, float inmax=0, int flag=0, int inlinear=0);
	int inside(int mousex,int mousey, int ox,int oy);
	virtual int process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current);
	virtual void show(float ox,float oy, int current);
};

//---------------------------------------------------------------------------------

class Slider:public Widget{
public:     Slider();
	virtual ~Slider();

	float min,max,range,value;
	int truncate, linear;
	static float texX0,texX1, texY0,texY1;

	virtual void init(char *s, int inID, int inindex, int intype,
		     float inx, float iny, float inwidth, float inheight,
			  float invalue, float inmin, float inmax, int flag, int inlinear);
	virtual void show(float ox,float oy, int current);
	virtual int process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current);
};

//---------------------------------------------------------------------------------

class Toggle:public Widget{
public:      Toggle();
	virtual ~Toggle();

	static float texX0,texX1, texY0,texY1;

	virtual void init(char *s, int inID, int inindex, int intype,
	        float inx, float iny, float inwidth, float inheight,
			  float invalue, float inmin, float inmax, int flag, int inlinear);
	virtual void show(float ox,float oy, int current);
	virtual int process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current);
};

//---------------------------------------------------------------------------------

class Button:public Widget{
public:      Button();
	virtual ~Button();

	static float texX0,texX1, texY0,texY1;

	virtual void init(char *s, int inID, int inindex, int intype,
		     float inx, float iny, float inwidth, float inheight,
			  float invalue, float inmin, float inmax, int flag, int inlinear);
	virtual void show(float ox,float oy, int current);
	virtual int process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current);
};

//---------------------------------------------------------------------------------

class Back:public Button{
public:     Back();
	virtual ~Back();

	static float texX0,texX1, texY0,texY1;

	virtual void init(char *s, int inID, int inindex, int intype,
		     float inx, float iny, float inwidth, float inheight,
			  float invalue, float inmin, float inmax, int flag, int inlinear);
	virtual void show(float ox,float oy, int current);
	virtual int process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current);
};

//---------------------------------------------------------------------------------

class Title:public Widget{
public:     Title();
	virtual ~Title();

	static float texX0,texX1, texY0,texY1;

	virtual void init(char *s, int inID, int inindex, int intype,
		     float inx, float iny, float inwidth, float inheight,
			  float invalue, float inmin, float inmax, int flag, int inlinear);
	virtual void show(float ox,float oy, int current);
	virtual int process(Key *key, Mouse *mouse, float ox, float oy, float *outValue, int current);
};

//---------------------------------------------------------------------------------

#define maxa 50
enum widgetTypes {SLIDER,TOGGLE,BUTTON,BACK,TITLE};
#define maxDialogDepth 19

class Dialog{
public:     Dialog();
	virtual ~Dialog();

	int ID, nitems, activeID, currentIndex, saveView, refresh;
	float activeValue, ox, oy;
	int dialogStack[maxDialogDepth], ndialogPushed;
	float fadedir, fade, inten;
	float dragx1, dragy1, dragx0,dragy0, lastox, lastoy;
	int x,y, sliderWidth,sliderHeight, sliderYinc, buttonWidth,buttonHeight, buttonYinc;
	int hasColor,hasLighting;

	Widget *controls[maxa];
	Text *text;
	Hw *hw;

	void clear(void);
	int control(int type, char *label, int ID,
				   float value=0, float min=0,float max=0, int truncate=0, int linear=0);

	void check(void);
	void show(void);
	void service(void);
	void init(int ID, int inpush);
	void push(void);
	void pop(void);
	void run(void);
	void dialog_init_standard(int ID, int mode);
	void dialog_do_standard(void);
	void on(void);
	void off(void);
};

void drawQ(Texture *tex, float x, float y, float x2, float y2,
		   float texX0,float texY0, float texX1,float texY1);

//---------------------------------------------------------------------------------------

#endif