//
#include "effect.h"
#include "text.h"

//------------------------------------------------------------------------

Effect::Effect(){
	sphereT=0;
	nobjects=11;
	for (int i=0; i<12; i++) {enables[i]=1;}
	//enables[2]=0; enables[3]=0;
	for (    i=0; i< 6; i++)     vars[i]=1;
};
Effect::~Effect(){};

//------------------------------------------------------------------------

void Effect::annot(Text *ann){
	if (hw->screen.capture) return;
	if ((hw->screen.viewport->camera.place.pos.y+hw->screen.viewport->camera.eyey)<dispman.dos[7].place.pos.y) {// underwater?
		if (groundM.inMesh(hw->screen.viewport->camera.place.pos.x,hw->screen.viewport->camera.place.pos.z)) {// on mesh?
			glEnable(GL_BLEND);
			set_blend(2,3);
			zColor4f(0,1,1,.2);
			glDepthMask(GL_FALSE); 
			hw->screen.rect();
			glDisable(GL_BLEND);
			glDepthMask(GL_TRUE);
		}
	}
	//for (int i=0; i<16; i++) {
	//	char s[80];
	//	sprintf(s,"%d : %.3f",i,hw->screen.viewport->camera.model[i]);
	//	ann->print(s,100,(16-i)*20,10);
	//}
}

//------------------------------------------------------------------------

void Effect::close(void){
	hw->sound.free_sample(satelliteSample);
	cloud1M.close();
	cloud2M.close();
	groundM.close();
	sphereM.close();
	waterM.close();
}

//------------------------------------------------------------------------

void Effect::do_teleport(Text *ann) {
	static int lastoption=hw->pref.option;
	static float lastteleporttime=0;
	float s;
	int option=-1;

	groundM.collide(&dispman.dos[8].place, &airPH);
	dispman.dos[8].place.move();
	hw->sound.set_position(hw->sound.sampleChannel,dispman.dos[8].place.pos.x,
																  dispman.dos[8].place.pos.y,
																  dispman.dos[8].place.pos.z);
	s=sinf(hw->time.time*pi2)*.5+.5;  // vary size and freq
	billBB.sizex=(s*4+1);
	billBB.sizey=billBB.sizex;
	hw->sound.lastChannel=hw->sound.sampleChannel;
	//hw->sound.set_frequency(s*4000+9000);
	hw->sound.set_frequency(s*10000+7000);

	if ((hw->time.realTime-lastteleporttime)>5) {
		if (length(hw->screen.viewport->camera.place.pos-dispman.dos[8].place.pos)<6) {// close enough?
			lastteleporttime=hw->time.realTime;
			while ((option==-1)||(option==lastoption)) option=rndn(maxoption+1); // new option
			lastoption=option;
			hw->pref.option=option;
			set_option(ann);
		}
	}
}

//------------------------------------------------------------------------

void Effect::draw(Text *ann){
	dispman.reset();
	
	if (enables[0]) dispman.draw(0); // ground
	if (enables[1]) dispman.draw(1); // rough color
	if (enables[2]) dispman.draw(2); // detail
	if (enables[3]) dispman.draw(3); // shadow
	if ((enables[6])&&(hw->screen.viewport->camera.place.movemode!=LOOKFROM)) {
		float w,h;

		dispman.draw(6); // sphere
		glEnable(GL_ALPHA_TEST);
		zColor3f(1,1,1);
		sphereT->bind();
		glPushMatrix();{
			Place *place=&dispman.dos[6].place; // satellite

			glTranslatef(place->pos.x,
				          place->pos.y,
							 place->pos.z);
			glBegin(GL_QUADS); // antennaes
				w=.15; h=10; // vert
				glTexCoord2f(0,0); glVertex2f(-w,-h);
				glTexCoord2f(1,0); glVertex2f( w,-h);
				glTexCoord2f(1,1); glVertex2f( w, h);
				glTexCoord2f(0,1); glVertex2f(-w, h);

				w=10; h=.15; // horiz
				glTexCoord2f(0,0); glVertex2f(-w,-h);
				glTexCoord2f(1,0); glVertex2f( w,-h);
				glTexCoord2f(1,1); glVertex2f( w, h);
				glTexCoord2f(0,1); glVertex2f(-w, h);

				glTexCoord2f(0,0); glVertex3f(0,-h,-w); // z
				glTexCoord2f(1,0); glVertex3f(0,-h, w);
				glTexCoord2f(1,1); glVertex3f(0, h, w);
				glTexCoord2f(0,1); glVertex3f(0, h,-w);

				glEnd();
			glDisable(GL_ALPHA_TEST);

			glPointSize(2);
			glDisable(GL_TEXTURE_2D);
			glBegin(GL_POINTS); {// lights
				float f=(((int)hw->time.time)%2);
				glColor3f(f,  0,0); glVertex3f( 3.536,0, 3.536);
				                    glVertex3f(-3.536,0,-3.536);
				glColor3f(0,1-f,0); glVertex3f( 3.536,0,-3.536);
										  glVertex3f(-3.536,0, 3.536);
				}
				glEnd();
				glEnable(GL_TEXTURE_2D);
		}
			glPopMatrix();
		Hw::get()->pref.ntris+=10;
	}

	if (enables[9]) dispman.draw(9); // cube
	if (enables[10]) dispman.draw(10); // moon

	if (hw->screen.viewport->camera.place.pos.y>dispman.dos[7].place.pos.y)
		if (enables[7]) dispman.draw(7); // water

	if (hw->pref.universe<4) if (enables[5]) dispman.draw(5); // top layer clouds

	if (enables[4]) dispman.draw(4); // bottom layer clouds

	if (hw->screen.viewport->camera.place.pos.y<=dispman.dos[7].place.pos.y)
		if (enables[7]) dispman.draw(7); // water

	if (enables[8]) dispman.draw(8); // teleport
	dispman.unset();

	//fallP.draw();
	//fogP.draw();
}

//------------------------------------------------------------------------

void Effect::end_frame(Text *ann){
	Place    *place=&hw->screen.viewport->camera.place;

   if (hw->time.time>1) {
		if ( (sign((place->    pos.y+hw->screen.viewport->camera.eyey)-dispman.dos[4].place.pos.y) != // just thru cloud layer 1?
				sign((place->lastpos.y+hw->screen.viewport->camera.eyey)-dispman.dos[4].place.pos.y)) ) {	
			hw->sound.play_type("sounds/whoosh2811.wav",FSOUND_MONO);
			hw->sound.set_volume(.25);
			hw->sound.set_frequency(120000);
		}
		if (hw->pref.universe<4) {
			if ( (sign((place->    pos.y+hw->screen.viewport->camera.eyey)-dispman.dos[5].place.pos.y) != // just thru cloud layer 2?
					sign((place->lastpos.y+hw->screen.viewport->camera.eyey)-dispman.dos[5].place.pos.y)) ) {	
				hw->sound.play_type("sounds/whoosh2811.wav",FSOUND_MONO);
				hw->sound.set_volume(.2);
				hw->sound.set_frequency(140000);
			}
		}
	}
	hw->   screen.viewport->camera.place.lastpos=
	   hw->screen.viewport->camera.place.pos;
}

//------------------------------------------------------------------------

void Effect::ground_update(void) { // shape/height/tess
	static int init=0;
	int i;

	if (init==0) {
		vars[0]=.5;
		vars[1]=.45;
		vars[2]=.25;
		vars[3]=175;
		init=1;
	}

	waterM.bumpgen();
	groundM.nodesx=hw->pref.tess;
	groundM.nodesy=groundM.nodesx;
	groundM.init();
	switch(hw->pref.option){
		case 0: groundM.shapeclipy1=150; // atlantis
			     groundM.shape(RINGS,    .05*hw->pref.worldSize*hw->pref.page,-25,1.5); break; // atlantis
		case 1: groundM.shape(SINBUMPS,.075*hw->pref.worldSize*hw->pref.page,  0,  1); break; // peru
		case 2: groundM.shape(COSBUMP,-.075*hw->pref.worldSize*hw->pref.page,-10,  1); break; // lake
		case 3: // everest
			groundM.shape(COSBUMP, .15*hw->pref.worldSize*hw->pref.page,-50,1);
			for (i=0; i<9; i++) {
				//groundM.depress(vars[0]*groundM.nodesx,vars[1]*groundM.nodesy,vars[2]*groundM.nodesx, -vars[3]);
				groundM.depress((rndn(.3)+.35)*groundM.nodesx,(rndn(.3)+.35)*groundM.nodesy,(rndn(.2)+.05)*groundM.nodesx, -(rndn(90)+10));
			}
			break;
		case 4: // canyon
			if (bump.load("images/grandCanyon128.bmp",1,0)) log_message("Cant make canyon bump **********");
			else groundM.init(&bump,.75*hw->pref.page,-125); break;
		case 5: // playground
			bump.make(1,128,60,0);
			groundM.init(&bump,1*hw->pref.page,-30); break;
		case 6: // earth surface
			if (bump.load("images/earth_256.bmp",   1,0)) log_message("Cant make earth bump **********");
			else groundM.init(&bump,.33*hw->pref.page,-90); break;
		case 7: // perlin
			if (bump.load("images/perlin.bmp",      1,0)) log_message("Cant make perlin bump **********");
			else groundM.init(&bump,.75*hw->pref.page,-80); break;
		case 8: // neptune surface
			if (bump.load("images/neptune2_256.bmp",1,0)) log_message("Cant make neptune bump **********");
			else groundM.init(&bump,.5*hw->pref.page,-110); break;
		case 9: // cloud
			if (bump.load("images/cloudbw2128.bmp", 1,0)) log_message("Cant make cloud bump **********");
			else groundM.init(&bump,.75*hw->pref.page,-80); break;
		case 10: // satin
			if (bump.load("images/satin.bmp", 1,0)) log_message("Cant make satin bump **********");
			else groundM.init(&bump,.5*hw->pref.page,-80); break;
	}
	groundM.calc_shadow();
	if (dispman.nobjects>=7) dispman.dos[6].place.moveCenter.y=groundM.maxheight+10;
}

//-------------------------------------------------------------------------

void Effect::init(Text *ann){
	int i=0;
	float ws,meter;

	log_message("Init effect...");

	reset(ann);
	ws   =hw->pref.worldSize;
	meter=hw->pref.meter;

	for (i=0; i<4; i++) { // program all cameras
		hw->screen.viewports[i].camera.place.moveRange. set_xyz(350,60,350);  // set camera movement
		hw->screen.viewports[i].camera.place.moveCenter.set_xyz(0,150,0);
		hw->screen.viewports[i].camera.place.spinRange. set_xyz(70,720,0);
		hw->screen.viewports[i].camera.place.moverate=1;
		switch(i) {// set movemode
			case 0:
				if (hw->pref.skipSplash) hw->screen.viewports[i].camera.place.movemode=MANUAL;
				else                     hw->screen.viewports[i].camera.place.movemode=LOOKAT;
				break;
			case 1: hw->screen.viewports[i].camera.place.movemode=LOOKAT; break;
			case 2: hw->screen.viewports[i].camera.place.movemode=LOOKFROM; break;
			case 3: hw->screen.viewports[i].camera.place.movemode=ORBIT; break;
		}
		hw->screen.viewports[i].camera.place.otherPlace=&dispman.dos[6].place; // @satelite
	}

	spacePH.gravity.   set_xyz(        0,  0*meter,        0); // physics ----------------------
	spacePH.movedrag.  set_xyz( .4*meter, .4*meter, .4*meter); // auto-thruster drag!
	spacePH.friction.  set_xyz(  0*meter,  0*meter,  0*meter);
	spacePH.rotatedrag.set_xyz(  7*meter,  7*meter,  7*meter);
	spacePH.spdLimit=300;

	airPH.gravity.   set_xyz(     0,-9.80665*meter,        0); // 32 f/s
	airPH.movedrag.  set_xyz( .5*meter, .5*meter, .5*meter);//.75
	airPH.friction.  set_xyz(  0*meter,  0*meter,  0*meter);
	airPH.rotatedrag.set_xyz(  7*meter,  7*meter,  7*meter);
	airPH.spdLimit=200;

	groundPH.gravity.   set_xyz(        0,  0*meter,        0);
	groundPH.movedrag.  set_xyz(1.5*meter,1.5*meter,1.5*meter);
	groundPH.friction.  set_xyz(  .5*meter,  .5*meter,  .5*meter);
	groundPH.rotatedrag.set_xyz(  7*meter,  7*meter,  7*meter);
	groundPH.spdLimit=50;

	waterPH.gravity.   set_xyz(       0, 4*meter,       0);
	waterPH.movedrag.  set_xyz( 3*meter, 3*meter, 3*meter);
	waterPH.friction.  set_xyz(       0,       0,       0);
	waterPH.rotatedrag.set_xyz(10*meter,10*meter,10*meter);
	waterPH.spdLimit=50;

	log_message("Init effect meshes...");  // meshes ---------------------------
	cloud1M.newmesh(4,4, ws*1.5,ws*1.5);
	cloud2M.newmesh(cloud1M.nodesx,cloud1M.nodesy, cloud1M.sizex,cloud1M.sizey);
	waterM.newmesh(8,8,ws*1,ws*1);
	sphereM.newmesh(10,10, 1,1); sphereM.spherical_map(5); sphereM.normals=1;
	groundM.sizex=ws*1; groundM.sizey=groundM.sizex; groundM.cullStrip=1;
	sphere.size=30;

	//particle_init();

	hw->sound.sample=hw->sound.load_wav("sounds/alien_beacon.wav");//radio_static1.wav");
	hw->sound.set_loop(hw->sound.sample);
	hw->sound.play(hw->sound.sample,100,0,100);
	hw->sound.set_reserved();
	hw->sound.sampleChannel=hw->sound.lastChannel;

	satelliteSample=hw->sound.load_wav("sounds/radio_static1_7.wav"); // satellite sound
	hw->sound.set_loop(satelliteSample);
	hw->sound.play(satelliteSample,0,0,0);
	hw->sound.set_reserved();
	satelliteChannel=hw->sound.lastChannel;

	log_message("Init effect textures..."); // textures -------------------------------
	if (cloudT.  load_tex("images/cloudbw2128.bmp", GL_LUMINANCE_ALPHA, -1, 0, 0))  log_message("Cant make cloud texture **********");
	if (groundT. load_tex("images/dirt128.bmp",                GL_RGBA, -1, 0, 1))  log_message("Cant make dirt **********");
	if (groundaT.load_tex("images/dirta128.bmp",               GL_RGBA, -1, 0, 1))  log_message("Cant make dirta texture **********");
	if (grassT.  load_tex("images/grass.bmp",                  GL_RGBA, -1, 0, 1))  log_message("Cant make grass texture **********");
	if (detailT.make_tex(SAND, 256,256, GL_LUMINANCE_ALPHA, 1))                     log_message("Cant make detail texture **********");
	if (metalT.load_tex("images/metalplate128.bmp",            GL_RGBA, -1, 0, 1))  log_message("Cant make metal texture **********");
	if (moonT.load_tex("images/moonmercator_wt256.bmp", GL_LUMINANCE_ALPHA,255,0,1))log_message("Cant make moon texture **********");

	log_message("Init effect objects...");
	load_data();
	ground_update();
	init_test(15); // color cube
}

//------------------------------------------------------------------------

void Effect::init_test(float size) { // color cube
	int f,i;
	float textures[4][2]={0,0, 1,0, 1,1, 0,1};
	float normals[6][3]={-1, 0, 0, //l
		                   1, 0, 0, //r
								 0,-1, 0, // b
								 0, 1, 0, // t
								 0, 0,-1, // f 
								 0, 0, 1};// n
	float vertices[6][4][3]={-1,-1,-1, -1,-1, 1, -1, 1, 1, -1, 1,-1,
		                       1,-1, 1,  1,-1,-1,  1, 1,-1,  1, 1, 1,
									 -1,-1,-1,  1,-1,-1,  1,-1, 1, -1,-1, 1,
									 -1, 1, 1,  1, 1, 1,  1, 1,-1, -1, 1,-1,
		                       1,-1,-1, -1,-1,-1, -1, 1,-1,  1, 1,-1,
									 -1,-1, 1,  1,-1, 1,  1, 1, 1, -1, 1, 1};
	Face *face=new Face;

	if (face==0) bail("Can't make new test object3d");
	for (f=0; f<6; f++) {
		face->normal.set_dir(normals[f][0],normals[f][1],normals[f][2]);
		for (i=0; i<4; i++) {
			face->vertexs[i].set_xyz(vertices[f][i][0]*size,
				                      vertices[f][i][1]*size,
											 vertices[f][i][2]*size);
			face->vertexs[i].set_color(rndn(1),rndn(1),rndn(1));
			face->vertexs[i].set_texture(textures[i][0],textures[i][1]);
		}
		o.add_face(face);
	}
	delete face;
}

//------------------------------------------------------------------------------

void Effect::inspace(Place *place, Text *ann) {
	static float lastlandtime=0;
	float dist, lookdrag, radii2=32.5;
	Vector g,moonpt;

	moonpt=dispman.dos[10].place.pos-place->pos; // moon-camera delta
	dist=moonpt.length();  // dist
	moonpt.normalize();    // moon normal
	g=(moonpt/(dist*.002));
	//g=moonpt/(dist*dist*.00003*hw->pref.home);  // scale grav by dist
	if (dist<=radii2) {    // collision?
		if ((place->vel.length()>hw->pref.gruntMPS*2)&&((hw->time.realTime-lastlandtime)>1)) { // hit hard?
			lastlandtime=hw->time.realTime;
			place->shakeampl=1; // shake camera
			hw->sound.play2d("sounds/land.wav");
			hw->sound.set_frequency(rndn(7000)+7500);
			hw->screen.set_dim(0, .9, 1,0,0,.5); // fade full->0, in red
			switch((int)rndn(3)) {
				case 0: ann->hold("M = F / A !"); break;
				case 1: ann->hold("The Law of Inertia."); break;
				case 2: ann->hold("Objects in motion remain in motion unless forced otherwise."); break;
			}
		}
		g*=0;             // no grav if on/under ground
	}
	spacePH.gravity=g;
	if (dist>radii2) 
		if ((hw->pref.pause==0)&&(hw->pref.dialogs==0)) place->physics->apply_gravity(place);// apply grav

	g=place->pos-dispman.dos[10].place.pos; // -delta
	dist=g.length();
	g.normalize();
	if (dist<=radii2) { // hit moon?
		place->acc+=g;///////////////////////
		place->pos=dispman.dos[10].place.pos+g*radii2;
	}
	lookdrag=7;  // spindrag
	place->physics->rotatedrag.set_xyz(lookdrag,lookdrag,lookdrag);
}

//------------------------------------------------------------------------------

void Effect::key(Text *ann){
	switch(hw->key.key){
		case 'e': dispman.selObject=(dispman.selObject>=dispman.nobjects)?               0:dispman.selObject+1; break;
		case 'E': dispman.selObject=(dispman.selObject<=               0)?dispman.nobjects:dispman.selObject-1; break;

		case '@': case 'r': object2pref(); load_data(); ground_update(); break;

		case 'i': if (dispman.selObject!=0) dispman.dos[dispman.selObject-1].vis.image=hw->pref.image;
			break;

		case 'o': set_option(ann); break;

		case '<': case '>': case '(': case ')': case '-': case '=': case '_': case '+': 
			ground_update(); break;

		case 127: // jump
			hw->screen.viewport->camera.place.acc.y+=10*hw->pref.meter;
			hw->screen.viewport->camera.place.pos.y +=.1; // take off ground
			break;
	}
	if ((hw->key.key=='e')||(hw->key.key=='E')) {
		if (dispman.selObject<9)
			Hw::get()->ann->hold_multi(dispman.selObject,
			"No Object selected",
			"Edit Ground","Edit Ground Rough Color","Edit Ground Detail","Edit Ground Shadow",
			"Edit Cloud 1","Edit Cloud 2",
			"Edit Satellite","Edit Water");
		else
			Hw::get()->ann->hold_multi(dispman.selObject-9,"Edit Teleporter","Edit Cube","Edit Moon");
		object2pref();
	}

	if (hw->key.skey) {
		if ((hw->key.alt)&&(!hw->key.control)&&(!hw->key.shift)) { // alt bank
			switch(hw->key.skey){
				case GLUT_KEY_F1:  enables[ 0]=!enables[ 0]; ann->hold("Ground: %d", enables[ 0]); break;
				case GLUT_KEY_F2:  enables[ 1]=!enables[ 1]; ann->hold("PatchColor: %d", enables[ 1]); break;
				case GLUT_KEY_F3:  enables[ 2]=!enables[ 2]; ann->hold("Detail: %d", enables[ 2]); break;
				case GLUT_KEY_F4:  enables[ 3]=!enables[ 3]; ann->hold("Shadow: %d", enables[ 3]); break;
				case GLUT_KEY_F5:  enables[ 4]=!enables[ 4]; ann->hold("Cloud1: %d", enables[ 4]); break;
				//case GLUT_KEY_F6:  enables[ 5]=!enables[ 5]; ann->hold("Enable5: %d", enables[ 5]); break;
				case GLUT_KEY_F7:  enables[ 5]=!enables[ 5]; ann->hold("Cloud2: %d", enables[ 5]); break;
				case GLUT_KEY_F8:  enables[ 6]=!enables[ 6]; ann->hold("Satellite: %d", enables[ 6]); break;
				case GLUT_KEY_F9:  enables[ 7]=!enables[ 7]; ann->hold("Water: %d", enables[ 7]); break;
				case GLUT_KEY_F10: enables[ 8]=!enables[ 8]; ann->hold("Teleporter: %d", enables[ 8]); break;
				case GLUT_KEY_F11: enables[ 9]=!enables[ 9]; ann->hold("Cube: %d",enables[ 9]); break;
				case GLUT_KEY_F12: enables[10]=!enables[10]; ann->hold("Moon: %d",enables[10]); break;
			}
		}
		if ((!hw->key.alt)&&(hw->key.control)&&(!hw->key.shift)) { // control bank
			switch(hw->key.skey){
				case GLUT_KEY_F1: vars[0]*=0.95   ; ann->hold("Var0: %.3f",vars[0]); ground_update(); break;
				case GLUT_KEY_F2: vars[0]*=1.05263; ann->hold("Var0: %.3f",vars[0]); ground_update(); break;
				case GLUT_KEY_F3: vars[1]*=0.95   ; ann->hold("Var1: %.3f",vars[1]); ground_update(); break;
				case GLUT_KEY_F4: vars[1]*=1.05263; ann->hold("Var1: %.3f",vars[1]); ground_update(); break;
				case GLUT_KEY_F5: vars[2]*=0.95   ; ann->hold("Var2: %.3f",vars[2]); ground_update(); break;
				case GLUT_KEY_F6: vars[2]*=1.05263; ann->hold("Var2: %.3f",vars[2]); ground_update(); break;
				case GLUT_KEY_F7: vars[3]*=0.95   ; ann->hold("Var3: %.3f",vars[3]); ground_update(); break;
				case GLUT_KEY_F8: vars[3]*=1.05263; ann->hold("Var3: %.3f",vars[3]); ground_update(); break;
				case GLUT_KEY_F9: vars[4]*=0.95   ; ann->hold("Var4: %.3f",vars[4]); break;
				case GLUT_KEY_F10:vars[4]*=1.05263; ann->hold("Var4: %.3f",vars[4]); break;
				case GLUT_KEY_F11:vars[5]*=0.95   ; ann->hold("Var5: %.3f",vars[5]); break;
				case GLUT_KEY_F12:vars[5]*=1.05263; ann->hold("Var5: %.3f",vars[5]); break;
			}
		}
		if ((!hw->key.alt)&&(!hw->key.control)&&(!hw->key.shift)) { // plain
			switch(hw->key.skey){
				case GLUT_KEY_PAGE_UP: case GLUT_KEY_PAGE_DOWN: ground_update(); break;
			}
		}
	}
}

//------------------------------------------------------------------------

void Effect::load_data(void){
	float ws=hw->pref.worldSize;
	float a[11][28]={ //    visuals and placement info for objects
//alpha  blend   color       cull dpth fg/lt plyo texs mag         off       siz     x    y   z   p  h r  tx#
   0,0,  0,0,0, .65,.4,.25,1,2,   1,1, 1,0,  0,  1,    30,    30,    0,    0,1000,   0,   0,  0,  0, 0,0, 13,//ground
	0,0,  1,2,1,     1,1,1,1, 2,   1,0, 1,0,  1,  1,     2,     2,    0,    0,1000,   0,   0,  0,  0, 0,0, 14,//rough color
	0,0,  1,6,2,  .2,.2,.2,1, 2,   1,0, 1,0,  1,  1,.75*ws,.75*ws,    0,    0,1000,   0,   0,  0,  0, 0,0, 16,//detail/0.2
	0,0,  1,2,1,  .2,.2,.2,1, 2,   1,0, 1,0,  1,  1,     6,     6,.0075,.0075,1000,   0,   0,  0,  0, 0,0, 12,//shadow//8.4
	0,0,  1,2,1,  .4,.4,.4,1, 0,   1,0,1.5,0, 0,  1,     4,     4, .005, .005,1000,   0, 100,  0,  0,90,0, 12,//cloud1/2.3
	0,0,  1,2,1,  .4,.4,.4,1, 0,   1,0,1.5,0, 0,  1,     4,     4, .005, .005,1000,   0, 200,  0,  0, 0,0, 12,//cloud2
	0,0,  0,2,3, .5,.5,.5,.5, 2,   1,1, 1,1,  0,  1,     3,     2,    0,    0,  20, 100, 150,100,  0, 0,0, 17,//satellite
   0,0,  1,7,1,.2,.46,.52,1, 0,   1,0, 1,1,  0,  1,     4,     4, .005, .005,1000,   0,-110,  0,  0, 0,0, 12,//water/5.6/7.1
	1,.05,1,2,3,     1,1,0,1, 2,   1,0, 1,0,  0,  1,     1,     1,    0,    0,  50, 110, 150,100,  0, 0,0, 9,//teleport
	0,0,  0,2,3,     1,1,1,1, 0,   1,1, 1,0,  0,  1,  1.12,  1.12,   .5,   .5,  50,   0, 250,  0,  0, 0,0, 12,//cube
	0,0,  0,2,3,     1,1,1,1, 2,   1,1, 0,1,  0,  1,     2,     1,    0,    0,  50,   0, 300,  0,  0, 0,0, 18};//moon

	BaseObject *os  [11]={&groundM,&groundM, &groundM,&groundM,
		                   &cloud1M,&cloud1M,&sphereM,&waterM,
								 &billBB,&o, &sphere};

	log_message("Load object data...");
	if (dispman.nobjects==0) {
		for (int i=0; i<nobjects; i++) dispman.add(os[i]); //create objects pak (object+vis+place)
	}
	for (int i=0; i<nobjects; i++) { // stuff data ---------------------------------
		Place *p;
		Vis *v;

		p=&dispman.dos[i].place;
		v=&dispman.dos[i].vis;
		v->alphatest =a[i][0];		v->alphafunc =a[i][1];
		v->blend     =a[i][2];		v->blendFunc0=a[i][3];		v->blendFunc1=a[i][4];
		v->r         =a[i][5];		v->g         =a[i][6];		v->b         =a[i][7];		v->a=a[i][8];
		v->cull      =a[i][9];
		v->depthtest =a[i][10];		v->depthwrite=a[i][11];
		v->fog       =a[i][12];		v->lights    =a[i][13];		v->polyo     =a[i][14];
		v->textures  =a[i][15];
		v->texmagx   =a[i][16];		v->texmagy   =a[i][17];
		v->texoffx   =a[i][18];		v->texoffy   =a[i][19];

		dispman.size[i]=a[i][20];
		p->pos.x   =a[i][21];		p->pos.y   =a[i][22];		p->pos.z   =a[i][23];
		p->orient.x=a[i][24];		p->orient.y=a[i][25];		p->orient.z=a[i][26];
		p->physics =&airPH;
		v->image=a[i][27];
		v->tex=Texture::textures[v->image];
	}
	dispman.dos[6].place.moveRange. set_xyz(275,0,275); // make sphere orbit
	dispman.dos[6].place.moveCenter.set_xyz(0,150,0);
	dispman.dos[6].place.spinRange. set_xyz(0,0,0);
	dispman.dos[6].place.movemode=ORBIT;// pos   =sinf(time*moverate)*moveRange)+moveCenter;
	                                    // orient=sinf(time*spinrate)*spinRange)+spinCenter;
	dispman.dos[6].place.moverate=1.2;

	dispman.dos[7].place.movemode=ORBIT; // water cycle ----------------------------
	dispman.dos[7].place.moveRange. set_xyz(0, 20,0);
	dispman.dos[7].place.moveCenter.set_xyz(0,-60,0);
	dispman.dos[7].place.moverate=.25;
	dispman.dos[7].place.orient.x=.1;
	dispman.dos[7].place.orient.z=.1;
	dispman.dos[7].place.spinRange.set_xyz(0,0,0);
	dispman.dos[7].place.spinrate=0;

	dispman.dos[9].place.movemode=ORBIT; // cube ----------------------------
	dispman.dos[9].place.moveRange. set_xyz(0, 0,0);
	dispman.dos[9].place.moveCenter.set_xyz(0,400,0);
	dispman.dos[9].place.moverate=0;
	dispman.dos[9].place.spinRange.set_xyz(720,720,0);
	dispman.dos[9].place.spinrate=1.5;

	dispman.dos[10].place.moveRange. set_xyz(700,0,700); // make moon orbit -------
	dispman.dos[10].place.moveCenter.set_xyz(0,700,0);
	dispman.dos[10].place.spinRange. set_xyz(0,0,0);
	dispman.dos[10].place.movemode=ORBIT;
	dispman.dos[10].place.moverate=.05;
}

//------------------------------------------------------------------------------------

void Effect::object2pref(void) {
	Vis *vis;

	if (dispman.selObject) { // map object attr to pref
		vis=&dispman.dos[dispman.selObject-1].vis;
		hw->pref.blendFunc0=vis->blendFunc0;
		hw->pref.blendFunc1=vis->blendFunc1;
		hw->pref.a1        =vis->texmagx;
		hw->pref.a2        =vis->texmagy;
		hw->pref.r         =vis->r;
		hw->pref.g         =vis->g;
		hw->pref.b         =vis->b;
		hw->pref.a         =vis->a;
		hw->pref.image     =vis->image;
		//hw->pref.movemode  =dispman.dos[dispman.selObject-1].place.movemode;
		//vars[0]=dispman.dos[dispman.selObject-1].vis.r;/////////////////////
		//vars[1]=dispman.dos[dispman.selObject-1].vis.g;
		//vars[2]=dispman.dos[dispman.selObject-1].vis.b;
		//vars[3]=dispman.dos[dispman.selObject-1].vis.a;
	}
}

//-----------------------------------------------------------------------

void Effect::particle_init(void) {
	Particle p;
	float meter=hw->pref.meter;

	log_message("Init effect particles...");

	//------------------------------------------------
	p.interval=.075;    p.lifeTime=1.5;       p.lifeTimeRnd=0;
	p.size0=meter*.0001;p.r0=1;               p.g0=.5;          p.b0=.25;       p.alpha0=1;
   p.size1=meter*.01;  p.r1=.5;              p.g1=.5;          p.b1=.5;        p.alpha1=0;  
	p.sizeRnd=meter*0;  p.sizey=1;            p.length=0*meter; p.colorRnd=0;

	p.windx=   0*meter; p.dispersex=.0002*meter;  p.xi=0*meter;
	p.windy= .02*meter; p.dispersey=.0002*meter;  p.yi=0*meter;
	p.windz=   0*meter; p.dispersez=.0002*meter;  p.zi=0*meter; p.rollspeed=0;

	p.alphatest=1;      p.alphafunc=0;       p.depthtest=1;     p.depthwrite=0;
	p.blend=1;          p.blendFunc0=2;      p.blendFunc1=1;
	p.maxparticles=p.lifeTime/p.interval;    p.init=1;          p.tex=&groundT; fallP=p;
	//------------------------------------------------
	p.interval=.075;     p.lifeTime=2;         p.lifeTimeRnd=.1;
	p.size0=meter*.0001; p.r0=1;               p.g0=1;           p.b0=1;         p.alpha0=1;
   p.size1=meter*.005;  p.r1=1;	             p.g1=1;           p.b1=1;         p.alpha1=0; 
	p.sizeRnd=meter*0;   p.sizey=1;            p.length=0*meter; p.colorRnd=0;

	p.windx=   0*meter; p.dispersex=.00002*meter; p.xi=0*meter;
	p.windy= .01*meter; p.dispersey=.00002*meter; p.yi=0*meter;
	p.windz=   0*meter; p.dispersez=.00002*meter; p.zi=0*meter; p.rollspeed=1; 

	p.alphatest=1;     p.alphafunc=0;        p.depthtest=1;     p.depthwrite=0;
	p.blend=1;         p.blendFunc0=2;       p.blendFunc1=1;
	p.maxparticles=p.lifeTime/p.interval;    p.init=1;          p.tex=&groundT;  fogP=p;
}

//-----------------------------------------------------------------------

void Effect::reset(Text *ann){
	static int init=0;

	hw=Hw::get();
	log_message("Reset effect...");
	hw->pref.blendFunc0=2;
	hw->pref.blendFunc1=1;
	hw->pref.b1=.1*hw->pref.worldSize;
	hw->pref.b2=.5*hw->pref.worldSize;
	hw->pref.tess=30;
	hw->pref.image=13;
	hw->pref.option=7;
	hw->pref.universe=3;

	for (int i=0; i<12; i++) {enables[i]=1;}
	//enables[2]=0; enables[3]=0; // disable detail and shadow
	for (    i=0; i< 6; i++)     vars[i]=1;

	hw->screen.lightmap.type=4;
	hw->screen.lightmap.texmagx=.25;
	hw->screen.viewport->camera.place.reset();
	dispman.selObject=0;
	if (init) {
		hw->screen.lightmap.on=1;
		hw->pref.fullscreen=1;
		hw->pref.letterbox=.12;
		hw->pref.clock=0;
	} else init=1;
}

//------------------------------------------------------------------------

void Effect::set_option(Text *ann) {
	static int lastimage=hw->pref.image;
	static float saveletterbox=hw->pref.letterbox;

	if (hw->pref.option<10) ann->hold_multi(hw->pref.option,"Atlantis.proc","Peru.proc","Lake.proc","Everest.proc","Grand Canyon.data",
		                             "Playground.proc","Mercator Earth.bmp","Perlin.bmp", "Neptune.bmp","Cloud.bmp");
	else ann->hold_multi(hw->pref.option-10,"satin.bmp");

	hw->pref.page=rndnn(1)*.4+1; // rnd height
	ground_update();

	hw->sound.play2d("sounds/teleport.wav");
	hw->screen.viewport->camera.place.pos.y=125;
	hw->screen.viewport->camera.place.physics=&airPH;
	hw->screen.viewport->camera.place.lastPhysics=&airPH;
	while (hw->pref.image==lastimage) hw->pref.image=rndn(4)+13; // rnd texture
	lastimage=hw->pref.image;
	dispman.dos[0].vis.image=hw->pref.image;
	hw->pref.universe=rndn(12); // rnd skybox/star combo
	dispman.dos[0].vis.set_color(rndn(.7)+.3,
		                          rndn(.7)+.3,
										  rndn(.7)+.3); // rnd ground color
	dispman.dos[1].vis.set_color(rndn(.8)+.2,
		                          rndn(.8)+.2,
										  rndn(.8)+.2); // rnd patch color
	dispman.dos[3].vis.set_color(rndn(.4)+.1,
		                          rndn(.4)+.1,
										  rndn(.4)+.1); // rnd shadow color
	if (rndn(1)<.2) {
		hw->screen.lightmap.type=8; // vignette
		hw->screen.lightmap.texmagx=1;
		hw->screen.lightmap.texmagy=1;
		hw->screen.lightmap.size=85;
		hw->pref.letterbox=0;
	} else {
		hw->pref.letterbox=saveletterbox;
		hw->screen.lightmap.type=(int)rndn(2)*4;
		if (rndn(1)<.5) {
			if (rndn(1)<.5) hw->screen.lightmap.type+=(int)rndn(2);   //x
			else            hw->screen.lightmap.type+=(int)rndn(2)*2; //y
		}
		if (rndn(1)<.5) {
			if (rndn(1)<.5) hw->screen.lightmap.type+=(int)rndn(2)*16;//xgrat
			else            hw->screen.lightmap.type+=(int)rndn(2)*32;//ygrat
		}
		hw->screen.lightmap.texmagx=rndn(5)+.1;
		hw->screen.lightmap.texmagy=rndn(5)+.1;
	}
}

//-----------------------------------------------------------------------

void Effect::start_frame(Text *ann){
	Viewport *viewport=hw->screen.viewport;
	Place    *place=&viewport->camera.place;
	float lookdrag;

	if (hw->pref.ex) return;

	place->to=dispman.dos[6].place.pos;// camera lookat satellite ----------
	hw->sound.set_position(satelliteChannel,dispman.dos[6].place.pos.x, // align sound to satellite
														 dispman.dos[6].place.pos.y,
														 dispman.dos[6].place.pos.z);
	hw->sound.lastChannel=satelliteChannel;
	if (place->movemode==LOOKFROM) hw->sound.set_volume(.2);
	else                           hw->sound.set_volume( 1);

	generalPH=airPH; // objects -------------------------------------------
	if (dispman.selObject) { // tweak colors
		dispman.dos[dispman.selObject-1].vis.r=vars[0];
		dispman.dos[dispman.selObject-1].vis.g=vars[1];
		dispman.dos[dispman.selObject-1].vis.b=vars[2];
		dispman.dos[dispman.selObject-1].vis.a=vars[3];
	}
	dispman.update(); // edits and orbits
	sphere.tessdist=length(dispman.dos[10].place.pos-place->pos); // moon-camera delta

	if (groundM.inMesh(place->pos.x,place->pos.z)) test_mesh_zone(ann); // mesh zone effects

	if (place->pos.y >= dispman.dos[9].place.pos.y+20) { // in space?
		place->physics=&spacePH;
		inspace(place,ann);
	}
	else { // clouds or below, adjust grav
		float ht;

		ht=hw->screen.viewport->camera.place.pos.y/350;        //map camera y of 0->350 as 0 to 1
		ht=1-clampf(ht,0,1);                                   // clamp and invert 1->0
		airPH.gravity.set_xyz(0, -9.80665*hw->pref.meter*ht, 0); // grav func(height)

		ht=hw->screen.viewport->camera.place.pos.y/100;        //map camera y of 0->100 as 0 to 1
		ht=1-clampf(ht,0,1);                                   // clamp and invert 1->0
		lookdrag=ht*6+1;                                       // spindrag varies 1->7 as y 0->100
		place->physics->rotatedrag.set_xyz(lookdrag,lookdrag,lookdrag); // camera spindrag func(height) map 
	}

	if ((hw->pref.help==0)&&(hw->pref.dialogs==0)) place->physics->damp(place);// camera damping
	place->lastPhysics=place->physics;

	//fallP.create(0,1,0);	fallP.update();
	//fogP.create(1,1,0);	fogP.update();
}

//------------------------------------------------------------------------

void Effect::test_mesh_zone(Text *ann) {
	Viewport *viewport=hw->screen.viewport;
	Place    *place=&viewport->camera.place;
	static float lastunsplashtime=0, lastlandtime=0;

	do_teleport(ann);

	if ((place->physics==&groundPH)&&(hw->pref.stepVolume>0)&&
		(hw->pref.pause==0)&&(hw->pref.dialogs==0)) track_steps();

	// track camera movement -------------------
	if (place->pos.y < dispman.dos[7].place.pos.y) underwater(place); // below water?
	else
	{ // above water? -----------------------------------
		place->physics=&airPH;
		//hw->sound.set_master_volume(1);
		if (place->lastPhysics==&waterPH) { // just out of water? -------------------
			if ((hw->screen.viewport->camera.place.vel.length()*.075)>1) { ///// fast enough?
				hw->sound.play2d("sounds/waterout1.wav"); // out-splash
				hw->sound.set_frequency(rndn(4000)+20000);
			}
			if ((hw->time.realTime-lastunsplashtime)>1) {
				lastunsplashtime=hw->time.realTime+rndnn(2)*2;
				hw->sound.play2d("sounds/inh2o.wav"); // still-in splash
				hw->sound.set_frequency(rndn(3000)+4000);
			}
		}
	}
	//   if manual cam              in main viewport     do camera/mesh physics ------------
	if ((place->movemode==MANUAL)&&(viewport==&hw->screen.viewports[0]) ) {
		if ((groundM.collide(place, &groundPH))&&((hw->time.realTime-lastlandtime)>2)) { // test mesh, hit & hard?
			lastlandtime=hw->time.realTime;
			hw->screen.viewport->camera.place.acc.x+=rndn(10)-5;
			hw->screen.viewport->camera.place.acc.z+=rndn(10)-5;
			hw->screen.viewport->camera.place.shakeampl=1; // shake camera
			hw->sound.play2d("sounds/land.wav");
			hw->sound.set_frequency(rndn(6000)+8000);
			hw->screen.set_dim(0, .9, 1,0,0,.5); // fade full->0, in red
			switch((int)rndn(3)) {
				case 0: ann->hold("M = F / A !"); break;
				case 1: ann->hold("The Law of Inertia."); break;
				case 2: ann->hold("Objects in motion remain in motion unless forced otherwise."); break;
			}
		}
	}
}

//---------------------------------------------------------------------------

void Effect::track_steps(void) {
	static float disttravelled=0, laststeptime=0;
	static int stepside=1;
	Viewport *viewport=hw->screen.viewport;
	Place    *place=&viewport->camera.place;

	disttravelled*=1-(.1*hw->time.dt);
	disttravelled+=place->vel.length()*hw->time.dt;
	if ((disttravelled>2)&&((hw->time.realTime-laststeptime)>.25)) { // step sound?// 1.5
		laststeptime=hw->time.realTime;
		disttravelled=0;
		stepside=-stepside; // toggle left/right
		if (place->pos.y>dispman.dos[7].place.pos.y) hw->sound.play2d("sounds/step-dirt.wav");
		else                                         hw->sound.play2d("sounds/step-grass.wav");
		hw->sound.set_volume(hw->pref.stepVolume);
		hw->sound.set_pan(.5+hw->pref.stepPan*stepside);
		//hw->sound.play2d("sounds/step-marble.wav");
		hw->sound.set_frequency(rndn(4000)+8000);
	}
}

//-----------------------------------------------------------------------

void Effect::underwater(Place *place) {
	static float lastglubtime=0;

	place->physics=&waterPH;
	//hw->sound.set_master_volume(.75);
	if (place->lastPhysics==&airPH) { // just dropped in?
		switch((int)(hw->screen.viewport->camera.place.vel.length()*.07)) { // adj splash
			case 0: break; // no sound
			case 1: hw->sound.play2d("sounds/little_splash.wav"); // tiny splash
					  hw->sound.set_frequency(rndn(20000)+10000);
				break;
			default:
				hw->sound.play("sounds/splash3.wav", // project big splash sound ahead
										place->pos.x-hw->screen.viewport->camera.forwardx*10,
										place->pos.y-hw->screen.viewport->camera.forwardy*10,
										place->pos.z-hw->screen.viewport->camera.forwardz*10);
				hw->sound.set_frequency(rndn(20000)+10000);
				break;
		}
	}
	if (((hw->time.realTime-lastglubtime)>.5)&&(hw->sound.on)) {// underwater sound
		if ((place->pos.y+hw->screen.viewport->camera.eyey) < dispman.dos[7].place.pos.y) {
			lastglubtime=hw->time.realTime+rndnn(2)*4;
			hw->sound.play("sounds/water_blup.wav");
			hw->sound.set_volume(rndn(.7)+.3);
			hw->sound.set_frequency(rndn(4000)+4000);
		}
	}
}

//------------------------------------------------------------------------------

/*	switch(dispman.selObject){
		case 0: ann->hold("No Object selected"); break;
		case 1: ann->hold("Edit Ground"); break;
		case 2: ann->hold("Edit Ground Rough Color"); break;
		case 3: ann->hold("Edit Ground Detail"); break;
		case 4: ann->hold("Edit Ground Shadow"); break;
		case 5: ann->hold("Edit Cloud 1"); break;
		case 6: ann->hold("Edit Cloud 2"); break;
		case 7: ann->hold("Edit Satellite"); break;
		case 8: ann->hold("Edit Water"); break;
		case 9: ann->hold("Edit Teleporter"); break;
		case 10: ann->hold("Edit Cube"); break;
		case 11: ann->hold("Edit Moon"); break;
	}
}*/

//-------------------------------------------------------------------





