//
#include "screen.h"
#include "hardware.h"

//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------

Screen::Screen(){
	reset();
};
Screen::~Screen(){};

//------------------------------------------------------------------------

void Screen::reset(void) {
	windowID=-1; capture=0;
	width=500; height=500; windowAspect=1; fullscreen=1; 
	windowWidth=width; windowHeight=height;
	letterbox=0; letterboxBottom=0; letterboxHeight=windowHeight;
	nviewports=1; viewportset=1; currentViewport=0; viewport=&viewports[currentViewport];
	nearz=.1; farz=10; needblank=0;
	for (int i=0;i<4;i++) color[i]=0;
	dimr=dimg=dimb=dima=0; dimdir=0; dimtime=0; dimduration=0;
}

//------------------------------------------------------------------------

void Screen::bound_cameras(float mx, float my) {
	Place *place;
	for (int viewportn=0; viewportn<nviewports; viewportn++) {
		place=&viewports[viewportn].camera.place;
		if (place->pos.x < -mx) {place->pos.x=-mx; place->vel.x*=.2;}
		if (place->pos.z < -my) {place->pos.z=-my; place->vel.z*=.2;}
		if (place->pos.x >  mx) {place->pos.x= mx; place->vel.x*=.2;}
		if (place->pos.z >  my) {place->pos.z= my; place->vel.z*=.2;}
	}
}

//-------------------------------------------------------------------

void Screen::clear(void){
	if (Hw::get()->pref.blank|needblank|(capture==1)) glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
	else if (Hw::get()->pref.c2>.01) {
		glClear(GL_DEPTH_BUFFER_BIT);
		viewport=&viewports[nviewports];
		viewport->size_viewport(width,height,0);
		set_ortho();
		fade(Hw::get()->pref.c2*Hw::get()->time.dt*40);
		viewport=&viewports[currentViewport];
	} else glClear(GL_DEPTH_BUFFER_BIT);
	needblank=maxi(0,needblank-1);
}

//------------------------------------------------------------------------

void Screen::draw_dim(void) {
	float inten;

	if (Hw::get()->time.realTime>(dimtime+dimduration)) return;
	switch((int)dimdir) { // shape time sawtooth
	case 0: case 1: // 0->1 or 1->0
		inten=cos_interp(1-dimdir,dimdir,         (Hw::get()->time.realTime-dimtime)/ dimduration); break;
	case 2: case 3: // v shape 0->1->0 or 1->0->1
		inten=cos_interp(3-dimdir,dimdir-2, (fabs((Hw::get()->time.realTime-dimtime)/(dimduration*.5))-1)); break;
	}
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(dimr*inten,dimg*inten,dimb*inten,dima*inten);
	rect();
	glDisable(GL_BLEND);
}

//-----------------------------------------------------------------------------

void Screen::fade(float alpha) { // variable screen fade F11/F12 control
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(color[0],color[1],color[2],alpha);
	rect();
	glDisable(GL_BLEND);
	return;
}

//---------------------------------------------------------------------------

void Screen::init() {
	width =glutGet(GLUT_SCREEN_WIDTH);
	height=glutGet(GLUT_SCREEN_HEIGHT);
}

//---------------------------------------------------------------------------

void Screen::rect(void) {
	glDepthMask(GL_FALSE); 
	glPushAttrib(GL_POLYGON_MODE);
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		glBegin(GL_QUADS);
			glTexCoord2f(0,0); glVertex2f(0,0);
			glTexCoord2f(1,0); glVertex2f(width,0);
			glTexCoord2f(1,1); glVertex2f(width,height);
			glTexCoord2f(0,1); glVertex2f(0,height);
			glEnd();
		glPopAttrib();
	glDepthMask(GL_TRUE);
	Hw::get()->pref.ntris+=2;
}

//-----------------------------------------------------------------------------

void Screen::set_dim(float indir, float induration, float inr, float ing, float inb, float ina) {
	dimtime=Hw::get()->time.realTime;
	dimdir=indir;
	dimduration=induration;
	dimr=inr;
	dimg=ing;
	dimb=inb;
	dima=ina;
}

//--------------------------------------------------------------------

void Screen::setup(void) {
	letterboxBottom=letterbox*windowHeight; // calc letterbox
	letterboxHeight=windowHeight-2*letterboxBottom;
	viewport->size_viewport(windowWidth,letterboxHeight,letterboxBottom);
	if (capture==1) viewport->useAspect=windowAspect*viewport->viewportAspect;
	else            viewport->useAspect=             viewport->viewportAspect;
}

//---------------------------------------------------------------------

void Screen::set_ortho(void) {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, viewport->iwidth, 0, viewport->iheight, -1,1); // l-r-b-t-n-f
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//-----------------------------------------------------------------------------

void Screen::set_perspective(void){
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(viewport->camera.fov, viewport->useAspect, nearz, farz);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//------------------------------------------------------------------------

void Screen::set_screen(void){
	if (fullscreen) {
		glutFullScreen();
		glutSetCursor(GLUT_CURSOR_NONE);
	} else {
		glutReshapeWindow(Hw::get()->pref.windowWidth,Hw::get()->pref.windowHeight);
		glutSetCursor(GLUT_CURSOR_LEFT_ARROW);
	}
}

//------------------------------------------------------------------------

void Screen::set_nviewports(int n) {
	viewportset=n;
	switch(n) { // viewports left bot right top
		case 1: {float vps[1][4]={0,   0,   1,   1}; load_viewport(vps, 1);}	break;
		case 2: {float vps[2][4]={0,   0,   1,  .5,
	                             0,  .5,   1,   1}; load_viewport(vps, 2);}	break;
		case 3: {float vps[2][4]={ 0,   0,  .5,  1,
	                              .5,  0,  1,   1}; load_viewport(vps, 2);}	break;
		case 4: {float vps[3][4]={0,   0,   1,  .6,
	                             0,  .6,  .5,   1,
								       .5,  .6,   1,   1}; load_viewport(vps, 3);} break;
		case 5: {float vps[4][4]={0,   0,   1,.666,
	                             0,.666,.333,   1,
	                          .333,.666,.666,   1,
	                          .666,.666,   1,   1}; load_viewport(vps, 4);}	break;
	}
}

//-------------------------------------------------------------------

void Screen::load_viewport(float array[][4], int n) {
	nviewports=n;
	currentViewport=0;
	viewport=&viewports[currentViewport];

	for (int i=0; i<nviewports; i++)
		viewports[i].set_view_pct(array[i][0],array[i][1],array[i][2],array[i][3]);
	viewports[nviewports].set_view_pct(0,0,1,1);
}

//-----------------------------------------------------------------------------

void Screen::doc_letterbox(void) {
	if      (fabs(((float)Hw::get()->screen.width/(Hw::get()->screen.height*(1-2*Hw::get()->pref.letterbox))-1.77777) )<.03)
		Hw::get()->ann->hold("Letterbox: %3.0f %% (16:9 Widescreen)",Hw::get()->screen.letterbox*100);
	else if (fabs(((float)Hw::get()->screen.width/(Hw::get()->screen.height*(1-2*Hw::get()->pref.letterbox))-2.35) 	)< .1)
		Hw::get()->ann->hold("Letterbox: %3.0f %% (2.35:1 Theatre)", Hw::get()->screen.letterbox*100);
	else if (fabs(((float)Hw::get()->screen.width/(Hw::get()->screen.height*(1-2*Hw::get()->pref.letterbox))-1.33333) )<.03)
		Hw::get()->ann->hold("Letterbox: %3.0f %% (4:3 TV)",         Hw::get()->screen.letterbox*100);
	else
		Hw::get()->ann->hold("Letterbox: %3.0f %%",                  Hw::get()->screen.letterbox*100);
}

//-----------------------------------------------------------------------------

