//-------------------------------------------------------------------
//	File:		   starfield.cpp
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "starfield.h"

//---------------------------------------------------------------------

Starfield::Starfield() {
	listID=0;
	n=10; nlevel=1;
	blend=1; depthtest=0; depthwrite=0;
	type=1;	size=2; hollow=0;
	minc=.3; maxc=.7;
};
Starfield::~Starfield() {
	destroy();
};

//---------------------------------------------------------------------

void Starfield::init(void) {
	int i;
	float x,y,z, dist, sizeh=size*.5, hollowh;
   
	hw=Hw::get();

	if (hollow>size*.9) hollow=size*.9;
	hollowh=hollow*.5+.01;
	destroy();
	listID=glGenLists(1);
	if (listID==0) return;
	glNewList(listID,GL_COMPILE); // star points
		glBegin(GL_POINTS);
			for (i=0; i<n/nlevel; i++) {
				switch (type) {
					default: 
					case 0: // hollow center sphere
						dist=-1;
						while( (dist<hollowh*hollowh) || (dist>sizeh) ) {
							x=(rndn(2)-1)*sizeh;
							y=(rndn(2)-1)*sizeh;
							z=(rndn(2)-1)*sizeh;
							dist=(x*x+y*y+z*z);
						}
						break;
					case 1: // hollow center cube
						x=0; y=0; z=0;
						while( (fabs(x)<hollowh)&&
							    (fabs(y)<hollowh)&&
							    (fabs(z)<hollowh) ) {
							x=(rndn(2)-1)*sizeh;
							y=(rndn(2)-1)*sizeh;
							z=(rndn(2)-1)*sizeh;
						}
						break;
					case 2: // hollow center dome
						dist=-1;
						while( (dist<hollowh*hollowh) || (dist>sizeh) || (y<0) ) {
							x=(rndn(2)-1)*sizeh;
							y=(rndn(2)-1)*sizeh;
							z=(rndn(2)-1)*sizeh;
							dist=(x*x+y*y+z*z);
						}
						break;
				}
			if (rndn(1)<.1) glColor3f((minc+rndn(maxc)),(minc+rndn(maxc)),(minc+rndn(maxc))); // few colored
			else            glColor3f((maxc+rndn(minc)),(maxc+rndn(minc)),(maxc+rndn(minc))); // mostly whitish
			glVertex3f(x,y,z);
			}
			glEnd();	
		glEndList();
}

//----------------------------------------------------

void Starfield::draw(void) {
	int i;

	if (listID==0) return;

	glPushMatrix();
		glTranslatef(hw->screen.viewport->camera.place.pos.x,
						 hw->screen.viewport->camera.place.pos.y,
						 hw->screen.viewport->camera.place.pos.z);
		if (blend) {
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glEnable(GL_BLEND); // need for smoothing
		}
		if (depthtest)   glEnable(GL_DEPTH_TEST);
		if (!depthwrite) glDepthMask(GL_FALSE);

		for (i=0; i<nlevel; i++) {
			glRotatef(360/nlevel,0,1,0);
			glPointSize(i+1.);
			glCallList(listID);
		}

		if (blend) glDisable(GL_BLEND);
		if (depthtest) glDisable(GL_DEPTH_TEST);
		if (!depthwrite) glDepthMask(GL_TRUE);
		hw->pref.ntris+=n;
		glPopMatrix();
}

//---------------------------------------------------------------------

void Starfield::destroy(void) {
	char s[80];

	if (listID>0) {
		sprintf(s,"   Destroying starfield display list # %u...",listID);
		log_message(s);
		glDeleteLists(listID,1);
		listID=0;
	}
}

//---------------------------------------------------------------------
//---------------------------------------------------------------------
//---------------------------------------------------------------------

Starfield2::Starfield2() {
	size=2;
	blend=1; depthtest=1; depthwrite=0;
	n=100;	type=1;
	hollow=0; minc=.3; maxc=.7;
	bound1=1; bound0=-1;
	tex=0; sizei=.1;
};
Starfield2::~Starfield2() {
};

//---------------------------------------------------------------------

void Starfield2::init(void) {
	int i;
	float x,y,z, sizeh=size*.5, dist, hollowh;

	hw=Hw::get();

	if (n>=maxarray) n=maxarray;
	if (hollow>size*.9) hollow=size*.9;
	hollowh=hollow*.5+.01;

	for(i=0; i<n; i++) {
		switch (type) {
			default:
			case 0: // hollow center sphere
				dist=-1;
				while( (dist<hollowh*hollowh) || (dist>sizeh) ) {
					x=(rndn(2)-1)*sizeh;
					y=(rndn(2)-1)*sizeh;
					z=(rndn(2)-1)*sizeh;
					dist=(x*x+y*y+z*z);
				} break;
			case 1: // hollow center cube
				x=0; y=0; z=0;
				while( (fabs(x)<hollowh)&&
					   (fabs(y)<hollowh)&&
					   (fabs(z)<hollowh) ) {
					x=(rndn(2)-1)*sizeh;
					y=(rndn(2)-1)*sizeh;
					z=(rndn(2)-1)*sizeh;
				} break;
			case 2: // hollow center dome
				dist=-1;
				while( (dist<hollowh*hollowh) || (dist>sizeh) || (y<0) ) {
					x=(rndn(2)-1)*sizeh;
					y=(rndn(2)-1)*sizeh;
					z=(rndn(2)-1)*sizeh;
					dist=(x*x+y*y+z*z);
				}
				break;
		}
		if (rndn(1)<.1) {
			rs[i]=minc+rndn(maxc);
			gs[i]=minc+rndn(maxc);
			bs[i]=minc+rndn(maxc);
		} else {
			rs[i]=maxc+rndn(minc);
			gs[i]=maxc+rndn(minc);
			bs[i]=maxc+rndn(minc);
		}
		xs[i]=x;
		ys[i]=y;
		zs[i]=z;
		if (rndn(1)<.25) intenis[i]=rndnn(2)*rndn(20)+.5;
		else             intenis[i]=0;
		intens[i]=1;
	}
}

//--------------------------------------------------------------------------

void Starfield2::update(void) {
	float sizem;
	int i;

	bound1=size*.35;
	bound0=-bound1;
	sizem=bound1*2;
	for(i=0; i<n; i++) {
		if (!hw->pref.pause) {
			if      (xs[i]>(hw->screen.viewport->camera.place.pos.x+bound1)) xs[i]-=sizem;
			else if (xs[i]<(hw->screen.viewport->camera.place.pos.x+bound0)) xs[i]+=sizem;
			if      (ys[i]>(hw->screen.viewport->camera.place.pos.y+bound1)) ys[i]-=sizem;
			else if (ys[i]<(hw->screen.viewport->camera.place.pos.y+bound0)) ys[i]+=sizem;
			if      (zs[i]>(hw->screen.viewport->camera.place.pos.z+bound1)) zs[i]-=sizem;
			else if (zs[i]<(hw->screen.viewport->camera.place.pos.z+bound0)) zs[i]+=sizem;
		}
		if (intenis[i]) {
			intens[i]+=intenis[i]*hw->time.dt;
			switch(sign(intenis[i])) {
			case -1:
				if (intens[i]<0) {
					intens[i]=0;
					intenis[i]=-intenis[i];
				}
				break;
			case 1:
				if (intens[i]>1) {
					intens[i]=1;
					intenis[i]=-intenis[i];
				}
				break;
			}
		}
	}
}

//----------------------------------------------------

void Starfield2::brownian(void) {
	int i;
	float s=.001*hw->pref.worldSize, s2=s*.5;

	for(i=0; i<n; i++) {
		xs[i]+=xis[i]*hw->time.dt;
		ys[i]+=yis[i]*hw->time.dt;
		zs[i]+=zis[i]*hw->time.dt;
		if (rndn(1)<1*hw->time.dt) {
			switch((int(rndn(3)))){
				case 0: xis[i]=rndn(s)-s2; break;
				case 1: yis[i]=rndn(s)-s2; break;
				case 2: zis[i]=rndn(s)-s2; break;
			}
		}
	}
}

//----------------------------------------------------

void Starfield2::draw(void) {
	int i;
	float sizex=sizei, sizey=sizei;
	float rightxs,rightys,rightzs, upxs,upys,upzs;

	if (blend) {
		set_blend(blendFunc0,blendFunc1);
		glEnable(GL_BLEND);
	}
	if (depthtest) glEnable(GL_DEPTH_TEST);
	if (!depthwrite) glDepthMask(GL_FALSE);
		
	if ((hw->pref.textures)&&(tex)) {
		glEnable(GL_TEXTURE_2D);
		tex->bind();
	}

	rightxs=hw->screen.viewport->camera.rightx*sizex;
	rightys=hw->screen.viewport->camera.righty*sizex;
	rightzs=hw->screen.viewport->camera.rightz*sizex;
	upxs   =hw->screen.viewport->camera.upx*sizey;
	upys   =hw->screen.viewport->camera.upy*sizey;
	upzs   =hw->screen.viewport->camera.upz*sizey;

	for (i=0; i<n; i++) {
		zColor3f(rs[i]*intens[i],gs[i]*intens[i],bs[i]*intens[i]);
		glBegin(GL_QUADS);
			glTexCoord2f(0,0); glVertex3f(-rightxs-upxs+xs[i],-rightys-upys+ys[i],-rightzs-upzs+zs[i]);
			glTexCoord2f(1,0); glVertex3f( rightxs-upxs+xs[i], rightys-upys+ys[i], rightzs-upzs+zs[i]);
			glTexCoord2f(1,1); glVertex3f( rightxs+upxs+xs[i], rightys+upys+ys[i], rightzs+upzs+zs[i]);
			glTexCoord2f(0,1); glVertex3f(-rightxs+upxs+xs[i],-rightys+upys+ys[i],-rightzs+upzs+zs[i]);
			glEnd();
	}
	if (blend) glDisable(GL_BLEND);
	if (depthtest) glDisable(GL_DEPTH_TEST);
	if (!depthwrite) glDepthMask(GL_TRUE);
	if ((hw->pref.textures)&&(tex)) glDisable(GL_TEXTURE_2D);
	hw->pref.ntris+=n*2;
}

//---------------------------------------------------------------------






