#include "utility.h"

#include "hardware.h"

//-------------------------------------------------------------------

float clampf(float value, float min, float max) {
	if      (value<min) return min;
	else if (value>max) return max;
	return value;
}

//----------------------------------------------------------------

int clampi(int value, int min, int max) {
	if      (value<min) return min;
	else if (value>max) return max;
	return value;
}

//----------------------------------------------------------------

float cos_interp(float a, float b, float x) {
	float f;

	f=(1-cosf(x*pi1))*.5;
	return a*(1-f)+b*f;
}

//---------------------------------------------------------------------------------------

void delay(int milliseconds) {
	int now, then;

	now=get_time();
	then=now+milliseconds;
	while(get_time()<then) {;}
	return;
}

//-------------------------------------------------------------------------------
char *doc_blend(void) {
	static char s[80];

	sprintf(s,"%d,%d: ",Hw::get()->pref.blendFunc0,Hw::get()->pref.blendFunc1);
	switch(Hw::get()->pref.blendFunc0) {
		case 0: strcat(s,"GL_ZERO"); break;
		case 1: strcat(s,"GL_ONE");  break;
		case 2: strcat(s,"GL_SRC_ALPHA");           break;
		case 3: strcat(s,"GL_ONE_MINUS_SRC_ALPHA"); break;
		case 4: strcat(s,"GL_DST_ALPHA");           break;
		case 5: strcat(s,"GL_ONE_MINUS_DST_ALPHA"); break;
		case 6: strcat(s,"GL_DST_COLOR");           break;
		case 7: strcat(s,"GL_ONE_MINUS_DST_COLOR"); break;
		case 8: strcat(s,"GL_SRC_ALPHA_SATURATE");  break;
	}
	strcat(s," , ");
	switch(Hw::get()->pref.blendFunc1) {
		case 0: strcat(s,"GL_ZERO");      break;
		case 1: strcat(s,"GL_ONE");       break;
		case 2: strcat(s,"GL_SRC_ALPHA"); break;
		case 3: strcat(s,"GL_ONE_MINUS_SRC_ALPHA"); break;
		case 4: strcat(s,"GL_DST_ALPHA");           break;
		case 5: strcat(s,"GL_ONE_MINUS_DST_ALPHA"); break;
		case 6: strcat(s,"GL_SRC_COLOR");           break;
		case 7: strcat(s,"GL_ONE_MINUS_SRC_COLOR"); break;
	}
	return s;
}

//------------------------------------------------------------------

char *doc_universe(void) {
	static char s[80];
	Hw *hw=Hw::get();

	if (hw->pref.universe==0) return "No Universe";
	else {
		s[0]=0;
		if (hw->pref.universe&1) strcat(s,"FarStars");
		if (hw->pref.universe&2) {
			if (s[0]!=0) strcat(s," + ");
			strcat(s,"NearParticles");
		}
		if (hw->pref.universe&4) {
			if (s[0]!=0) strcat(s," + ");
			strcat(s,"Fixed Skybox");
		}
		if (hw->pref.universe&8) {
			if (s[0]!=0) strcat(s," + ");
			strcat(s,"Movable Skybox");
		}
	}
	return s;
}

//----------------------------------------------------------------------

void draw_billboard(float size) {
	glPushMatrix();
		glRotatef(Hw::get()->screen.viewport->camera.place.orient.y,0,1,0);
		glRotatef(Hw::get()->screen.viewport->camera.place.orient.x,1,0,0);
		glBegin(GL_QUADS);
			glTexCoord2f(0,0); glVertex2f(-size,-size);
			glTexCoord2f(1,0); glVertex2f( size,-size);
			glTexCoord2f(1,1); glVertex2f( size, size);
			glTexCoord2f(0,1); glVertex2f(-size, size);
			glEnd();
		glPopMatrix();
	Hw::get()->pref.ntris+=2;
}

//--------------------------------------------------------------------------

void draw_frame(float x0, float y0, float x1, float y1, float thick) {
	glLineWidth(thick);
	glBegin(GL_LINE_LOOP);
		glVertex2f(x0,y0);
		glVertex2f(x1,y0);
		glVertex2f(x1,y1);
		glVertex2f(x0,y1);
		glEnd();
	glLineWidth(2);
}

//-------------------------------------------------------------------------------

float frac(float f) {
	return f-(int)f;
}

//---------------------------------------------------------------------------------------

void guage(char *s, char *s2, float inx, float y, float value, float range) {
	float size=120, height=16, edge=3, x=inx;
	Vector color;
	Text label;

	if (value>range) {
		color.set_xyz(1,1,0);
		value=range;
	} else color.set_xyz(0,1,0);
	glDisable(GL_DEPTH_TEST);
	glColor3f(.5,.5,.5);
	glRectf(x,y,x+size,y+height);
	glColor3f(0,0,0);
	glRectf(x+2,y+2,x+size-2,y+height-2);
	glColor3f(color.x,color.y,color.z);
	glRectf(x+edge,y+edge,x+(value/range)*(size-2*edge)+edge,y+height-edge);

	x-=strlen(s)*label.size*label.xinc;
	y+=7;
	label.print(s,x,y,9);

	x=inx+size+10;
	label.print(s2,x,y,9);
	return;
}

//----------------------------------------------------------------------------------

int get_time(void){
	return glutGet(GLUT_ELAPSED_TIME);
}

//------------------------------------------------------------------------

float length(Vector v) {
	return (float) sqrt(v.x*v.x+v.y*v.y+v.z*v.z);
}

//------------------------------------------------------------------------

float lin_interp(float a, float b, float x) {
	return  a*(1-x)+b*x;
}

//---------------------------------------------------------------------------------------

int log_message(char *ins) {
	char s[200]={0},n[80]={0};
	static int nlines=0;

	if (Hw::get()->screen.windowID>=0) {                 // is glut timer active?
		int t=get_time();//glutGet(GLUT_ELAPSED_TIME);
		sprintf(s," %2d:%02d:%02d.%03d - ",(t/1440000), // format hours (23:59:59:999)
			                                (t/  60000)%60, // min.
													  (t/   1000)%60, // sec.
													   t         %999); // ms.
	}
	strcat(s,ins);                                     // prepend time to message
	strcat(s,"\n");                                    // append linefeed
	cout << s;                                         // console it
	if (nlines>5000) {                                   // limit file lines to 50
		if (nlines==5001) {
			cout << "Log Message Overrun.\n";
			nlines++;
		}
		return 1;
	}
	strcpy(n,Hw::get()->pref.title);                     // build file name "<app>.log"
	strcat(n,".log");
	ofstream file(n,(nlines==0)? ios::out : ios::app); // open
	if (file.bad()||(file==0)) return 1;               // bad?
	file.write((const char *) s, strlen(s));           // print
	file.close();
	nlines++;                                          // count
	return 0;
}

//-----------------------------------------------------------------------------
/*
void matmult(Camera *camera, float *inx, float *iny, float *inz) {
	float x,y,z;

	x=camera->rightx  **inx+
	  camera->upx     **iny+
	  camera->forwardx**inz;
	y=camera->righty  **inx+
	  camera->upy     **iny+
	  camera->forwardy**inz;
	z=camera->rightz  **inx+
	  camera->upz     **iny+
	  camera->forwardz**inz;
	*inx=x;
	*iny=y;
	*inz=z;
}
*/
//-------------------------------------------------------------------------------

float maxf(float value0, float value1) {
	if (value0>value1) return value0;
	else               return value1;
}

//--------------------------------------------------------------------

int maxi(int value0, int value1) {
	if (value0>value1) return value0;
	else               return value1;
}

//--------------------------------------------------------------------

float minf(float value0, float value1) {
	if (value0<value1) return value0;
	else               return value1;
}

//--------------------------------------------------------------------

int mini(int value0, int value1) {
	if (value0<value1) return value0;
	else               return value1;
}

//--------------------------------------------------------------------

void noop(void) {}

//--------------------------------------------------------------------

float rndn(float range) { // return (0->.99999999) * range
	//return (float)(((float)rand()/(RAND_MAX+1.))*range);
	return ((rand()/(RAND_MAX+1.f))*range);
}

//--------------------------------------------------------------------

float rndnn(int n) {	 //  rndn(1)* done n times
	float v=1;
	for (int i=0; i<n; i++)	v*=rndn(1);
	return v;
}

//----------------------------------------------------------------------------

void set_blend(int blend0, int blend1) {
	int blendFunc0[]={GL_ZERO,
					  GL_ONE,
					  GL_SRC_ALPHA,
					  GL_ONE_MINUS_SRC_ALPHA,
					  GL_DST_ALPHA,
					  GL_ONE_MINUS_DST_ALPHA,
					  GL_DST_COLOR,
					  GL_ONE_MINUS_DST_COLOR,
					  GL_SRC_ALPHA_SATURATE};
	int blendFunc1[]={GL_ZERO,
					  GL_ONE,
					  GL_SRC_ALPHA,
					  GL_ONE_MINUS_SRC_ALPHA,
					  GL_DST_ALPHA,
					  GL_ONE_MINUS_DST_ALPHA,
					  GL_SRC_COLOR,
					  GL_ONE_MINUS_SRC_COLOR};

	glBlendFunc(blendFunc0[blend0],blendFunc1[blend1]);
	return;
}

//----------------------------------------------------------------------

void set_polymode(Pref *pref) {
	int modes[3]={GL_FILL,GL_LINE,GL_POINT};

	glPolygonMode(GL_FRONT_AND_BACK,modes[pref->wireframe]);
}

//--------------------------------------------------------------------

int sign(float value) {
	if      (value>0) return  1;
	else if (value<0) return -1;
	else              return  0;
}

//----------------------------------------------------------------------------
// create sphere of size, tesselated by tesx/y,
//   with texcoord starting at texoff* - texmag*, (1,0 mag,off is "normal" mapping)
void solid_sphere(float size, int tesx, int tesy,
				      float texmagx, float texmagy,
				      float texoffx, float texoffy) {
	int aix, ix,iy, iy2;
	float rx,ry, cx, cy,cy2, dx,dy,dz, dx2,dy2,dz2;
	float fx,fy,fy2, texx, texy, texy2;
	if ((tesx<  3)||(tesy<  3)) return;
	if ((tesx>100)||(tesy>100)) return;
	rx=pi2/ tesx;
	ry=pi1/(tesy-1);
	for    (iy=0; iy<(tesy-1); iy++) {
		iy2=iy+1;
		fy =iy*ry;
		fy2=iy2*ry;
		dy = -cosf(fy );
		dy2= -cosf(fy2);
		cy =(float)iy /tesy;
		cy2=(float)iy2/tesy;
		texy =texoffy+texmagy*cy;
		texy2=texoffy+texmagy*cy2;

		glBegin(GL_TRIANGLE_STRIP);
			for (ix=0; ix<=tesx; ix++) {
				cx=(float)ix /tesx;
				aix=(ix==tesx) ? 0 : ix;
				fx=aix*rx;
				dx =sinf(fx)*sinf(fy);  dz =cosf(fx)*sinf(fy);
            dx2=sinf(fx)*sinf(fy2); dz2=cosf(fx)*sinf(fy2);
				texx=texoffx+texmagx*cx;

				glNormal3f(dx2,dy2,dz2); glTexCoord2f(texx, texy2); glVertex3f(dx2*size, dy2*size, dz2*size);
				glNormal3f(dx ,dy ,dz);  glTexCoord2f(texx, texy);  glVertex3f(dx *size, dy *size, dz *size);
			}
			glEnd();
	}
	return;
}

//------------------------------------------------------------------------------

void tex_gen(int map, int mode) {
	GLfloat sv[]={1,0,0,0},  // sv[0]=obj x mag   1=yshear 2=zmag  3 xtrans
		    tv[]={0,1,0,0},  // tv[0]=obj y shear 1=ymag   2=zmag  3 ytrans
			qv[]={0,0,0,.08};      // qv[3] /mag .08

	//if (map) return;
	if (mode==1) { // on
		if (map) {
			switch(map) {
			case 1:
				glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
				glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
				glEnable(GL_TEXTURE_GEN_S);
				glEnable(GL_TEXTURE_GEN_T);
				break;
			case 2:
				glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
				glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
				glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
				glTexGenfv(GL_S,GL_OBJECT_PLANE,sv);
				glTexGenfv(GL_T,GL_OBJECT_PLANE,tv);
				glTexGenfv(GL_Q,GL_OBJECT_PLANE,qv);
				glEnable(GL_TEXTURE_GEN_S);
				glEnable(GL_TEXTURE_GEN_T);
				glEnable(GL_TEXTURE_GEN_Q);
				break;
			case 3:
				glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
				glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
				glTexGenfv(GL_S,GL_OBJECT_PLANE,sv);
				glTexGenfv(GL_T,GL_OBJECT_PLANE,tv);
				glEnable(GL_TEXTURE_GEN_S);
				glEnable(GL_TEXTURE_GEN_T);
				break;
			}
		}
	} else {
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
		glDisable(GL_TEXTURE_GEN_Q);
	}
}

//------------------------------------------------------------------------

void v2matrix(Vector forward) {
	float m[16];
	Vector up(0,1,0),right;

	//up.set_xyz(0,1,0);
	forward.normalize();
	right=forward.cross_product(up);
	right.normalize();
	up=forward.cross_product(right);
	up.normalize();
	m[0]=right.x;
	m[1]=right.y;
	m[2]=right.z;
	m[3]=0;

	m[4]=up.x;
	m[5]=up.y;
	m[6]=up.z;
	m[7]=0;

	m[8] =forward.x;
	m[9] =forward.y;
	m[10]=forward.z;
	m[11]=0;

	m[12]=0;
	m[13]=0;
	m[14]=0;
	m[15]=1;
	glMultMatrixf(m);
}

//--------------------------------------------------------------------------

float wrap(float x, float min, float max) {
	float range=max-min;
	int n=x/range;
	if (x<0) n--;
	x-=n*range;
	return x;
}

//-----------------------------------------------------------------

// convert normalized x/y vector to radian rotation, CW relative to 12
// vector   clock   radian
//  0, 1      12         0
//  1, 0       3    1/2 pi
//  0,-1       6        pi
// -1, 0       9    1.5*pi
/*
float xy2r(float x,float y) {
	float r;
	if (fabs(x)<fabs(y)) {
		if (y) r=atan(x/y);
		else if (x>0) r= pih;
		else          r= pi1+pih;
		if (y<0)      r+=pi1;
	} else {
		if (x) r=pih-atan(y/x);
		else if (y>0) r= 0;
		else          r= pi1;
		if (x<0)      r+=pi1;
	}
	return r;
}*/
float xy2r(float x,float y) {
	return atan2(x,y);
}
// convert normalized x/y vector to radian rotation, CW relative to 12
// vector   clock   radian
//  0, 1      12         0
//  1, 0       3    1/2 pi
//  0,-1       6        pi
// -1, 0       9    1.5*pi
/*
float xy2r(float x,float y) {
	if (x>=0) return     acosf(y);
	else      return pi2-acosf(y);
}*/

//--------------------------------------------------------------------

void zColor3f(float r, float g, float b) {
	float f;
	switch(Hw::get()->pref.colorMode) {
		case 0:                 glColor3f(r,g,b); break; // normal
		case 1: f=(r+g+b)*.33;//glColor3f(f*.7,f*.2,0);
			                     glColor3f(f*.72,f*.43,f*.29); break; // sepia
		case 2: f=(r+g+b)*.33;  glColor3f(0,f*.35,f*.4);      break; // blu/grn
		case 3: f=(r+g+b)*.33;  glColor3f(0,f,0);             break; // grn
		case 4:                 glColor3f(1-r,1-g,1-b);       break; // inverse 1-c
		case 5: f=(r+g+b)*.33;  glColor3f(f,f,f); break; // b&w
	}
}

//--------------------------------------------------------------------

void zColor4f(float r, float g, float b, float a) {
	float f;
	switch(Hw::get()->pref.colorMode) {
		case 0:                 glColor4f(r,g,b,a); break; // normal
		case 1: f=(r+g+b)*.33;//glColor4f(f*.7,f*.2,0,a);
			                     glColor3f(f*.72,f*.43,f*.29); break; // sepia
		case 2: f=(r+g+b)*.33;  glColor4f(0,f*.35,f*.4,a);    break; // blu/grn
		case 3: f=(r+g+b)*.33;  glColor4f(0,f,0,a);           break; // grn
		case 4:                 glColor4f(1-r,1-g,1-b,a);     break; // inverse 1-c
		case 5: f=(r+g+b)*.33;//glColor4f(f,f,f,a);
			                     glColor4f(1,1,1,a); break; // b&w
	}
}

//--------------------------------------------------------------------------

void zLookAt(Vector *from, Vector *to, float *pitch, float *heading){
	Vector delta; // return eulers (pitch and heading) to orient to-from

	delta=*to-*from;
	delta.normalize();
	*heading=-(57.2958*xy2r(delta.x,                            -delta.z));
	*pitch  = (57.2958*xy2r(delta.y,sqrt(delta.x*delta.x+delta.z*delta.z)));
}

//--------------------------------------------------------------------------
/*
void zLookAt(Vector *from, Vector *to, float *pitch, float *heading){
	float x,y,z,len;

	x=to->x-from->x;
	y=to->y-from->y;
	z=to->z-from->z;
	len=sqrt(x*x+y*y+z*z);
	if (len<.00001) len=.00001;
	x/=len;
	y/=len;
	z/=len;
	*heading=-(57.2958*xy2r(x,          -z));
	*pitch=   (57.2958*xy2r(y,sqrt(x*x+z*z)));
}
*/
//--------------------------------------------------------------------------

void rotate_axes(Vector *a, Vector *b, float angle) { 
	float c=cosf(angle), s=sinf(angle);
	Vector temp=*a*c + *b*s;
	*b=*a*-s + *b*c;
	*a=temp;
}
// to pitch, call rotateAxes(Look, Up, angle).

//--------------------------------------------------------------------------

