//
#include "vector.h"

#include "main.h"

//------------------------------------------------------------------------

//Vector::Vector() {
	//set_xyz(0,0,0);
//};

Vector::Vector(float x, float y, float z) {
	set_xyz(x,y,z);
};

Vector::~Vector(){};
	
//----------------------------------------------------------------------

void Vector::set_xyz(float inx, float iny, float inz) {
	x=inx;
	y=iny;
	z=inz;
}

//----------------------------------------------------------------------

void Vector::normalize(void) {
	float len=length();
	if (len<.0001f) len=.0001f;
	len=1.f/len;
	x*=len;
	y*=len;
	z*=len;
}

//----------------------------------------------------------------------

float Vector::length(void) {
	return (float) sqrt(x*x+y*y+z*z);
}

//----------------------------------------------------------------------

Vector Vector::cross_product(Vector v) {
	Vector result;
	
	result.x=(y*v.z)-(z*v.y);
	result.y=(z*v.x)-(x*v.z);
	result.z=(x*v.y)-(y*v.x); 
	return result;
}

//-------------------------------------------------------------------

float Vector::dot_product(Vector v) {
	return x*v.x+
		    y*v.y+
			 z*v.z;
}

//-------------------------------------------------------------------

Vector operator+(const Vector& v1, const Vector& v2){
	Vector r=v1;
	r.x+=v2.x;
	r.y+=v2.y;
	r.z+=v2.z;
	return r;
}

//----------------------------------------------------------------------

Vector operator-(const Vector& v1, const Vector& v2){
	Vector r=v1;
	r.x-=v2.x;
	r.y-=v2.y;
	r.z-=v2.z;
	return r;
}

//----------------------------------------------------------------------

Vector operator*(const Vector& v1, const float& f){
	Vector r=v1;
	r.x*=f;
	r.y*=f;
	r.z*=f;
	return r;
}

//----------------------------------------------------------------------

Vector operator/(const Vector& v1, const float& f){
	Vector r=v1;
	float f2;

	if (f<.0001) f2=.0001;
	else         f2=f;
	r.x/=f2;
	r.y/=f2;
	r.z/=f2;
	return r;
}

//----------------------------------------------------------------------

const Vector& Vector::operator+=(const Vector& v2) {
	x+=v2.x;
	y+=v2.y;
	z+=v2.z;
	return *this;
}

//----------------------------------------------------------------------

const Vector& Vector::operator-=(const Vector& v2) {
	x-=v2.x;
	y-=v2.y;
	z-=v2.z;
	return *this;
}

//----------------------------------------------------------------------

const Vector& Vector::operator*=(const float& f) {
	x*=f;
	y*=f;
	z*=f;
	return *this;
}

//----------------------------------------------------------------------

const Vector& Vector::operator/=(const float& f) {
	float f2;
	if (f<.0001) f2=.0001;
	else         f2=f;
	x/=f2;
	y/=f2;
	z/=f2;
	return *this;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------

